@@ -2,7 +2,7 @@ import { css } from '@emotion/css';
 
 import { GrafanaTheme2 } from '@grafana/data';
 import { t } from '@grafana/i18n';
-import { Drawer, Icon, Stack, Text, Tooltip, Tab, TabsBar, useStyles2 } from '@grafana/ui';
+import { Drawer, Stack, Text, Tab, TabsBar, useStyles2 } from '@grafana/ui';
 import { contextSrv } from 'app/core/services/context_srv';
 
 import { useQueryLibraryContext } from '../../features/explore/QueryLibrary/QueryLibraryContext';
@@ -32,12 +32,6 @@ export function QueryLibraryDrawer() {
         title={
           <Stack alignItems="center">
             <Text element="h3">{t('query-library.drawer.title', 'Saved queries')}</Text>
-            <Tooltip
-              placement="right"
-              content={t(`query-library.drawer.tooltip`, 'Right now, each organization can save up to 1000 queries')}
-            >
-              <Icon name="info-circle" />
-            </Tooltip>
           </Stack>
         }
         onClose={() => closeDrawer(false)}
@@ -55,7 +49,7 @@ export function QueryLibraryDrawer() {
               onChangeTab={() => onTabChange(QueryLibraryTab.FAVORITES)}
               disabled={!!newQuery}
             />
-            {!contextSrv.hasRole('Viewer') && (
+            {contextSrv.isEditor && (
               <Tab
                 label={t('query-library.tabs.recent', 'Recent')}
                 active={activeTab === QueryLibraryTab.RECENT}
